library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

entity p16_fir_pipeline is port(
	clk : in std_logic;
	data_in : in std_logic_vector(11 downto 0); -- 12 Bit unsigned
	data_out : out std_logic_vector(11 downto 0));
end p16_fir_pipeline;

architecture behav of p16_fir_pipeline is

constant filterordnung : integer := 16;

type koeffizienten_rom_8 is array (0 to 7) of integer range -2**11 to 2**11-1;
-- LP Barthann F_c=0.1, F_pb=0.18, F_sb=0.2
constant lp_8_koeffizienten : koeffizienten_rom_8 :=(57,291,682,1018,1018,682,291,57);

type koeffizienten_rom_16 is array (0 to 15) of integer range -2**11 to 2**11-1;
-- LP Barthann F_c=0.1, F_pb=0.18, F_sb=0.2
constant lp_16_koeffizienten : koeffizienten_rom_16 :=(-10,-26,-22,42,193,415,648,807,807,648,415,193,42,-22,-26,-10);
-- BP Barthann F_c=0.1, F_c2=0.25, F_sb=0.18, F_pb=0.2, F_pb2=0.25, F_sb2=0.27
constant bp_16_koeffizienten : koeffizienten_rom_16 :=(3,3,78,58,-388,-761,-88,1117,1117,-88,-761,-388,58,78,3,3);

type koeffizienten_rom_64 is array (0 to 63) of integer range -2**11 to 2**11-1;
-- LP Barthann F_c=0.1, F_pb=0.18, F_sb=0.2
constant lp_64_koeffizienten : koeffizienten_rom_64 :=(
1,0,-1,-3,-5,-5,-3,3,10,15,15,7,-8,-24,-35,-33,
-14,17,50,71,66,29,-34,-103,-150,-145,-67,84,290,511,698,809,
809,698,511,290,84,-67,-145,-150,-103,-34,29,66,71,50,17,-14,
-33,-35,-24,-8,7,15,15,10,3,-3,-5,-5,-3,-1,0,1);
-- BP Barthann F_c=0.1, F_c2=0.25, F_sb=0.18, F_pb=0.2, F_pb2=0.25, F_sb2=0.27
constant bp_64_koeffizienten : koeffizienten_rom_64 :=(
-1,-1,2,4,1,1,7,4,-17,-24,-2,8,-10,3,57,59,
-18,-53,-6,-20,-122,-95,110,192,44,18,221,109,-544,-873,-88,1045,
1045,-88,-873,-544,109,221,18,44,192,110,-95,-122,-20,-6,-53,-18,
59,57,3,-10,8,-2,-24,-17,4,7,1,1,4,2,-1,-1);

signal sample_in: integer range -2**11 to 2**11-1:=0;
type sum_ram is array (0 to filterordnung-1) of integer range -2**31 to 2**31-1;	
signal sum_shift_ram : sum_ram :=(others => 0);

signal sample_out_post_scale : integer range -2**31 to 2**31-1:=0;

begin

process begin
	wait until rising_edge(CLK);
	
	sample_in <= to_integer(unsigned(data_in)) - 2**11; --unsigned input
	
	sum_shift_ram(0) <= sample_in * lp_16_koeffizienten(0);
	for I in 1 to filterordnung-1 loop
		sum_shift_ram(I) <= sum_shift_ram(I-1) + lp_16_koeffizienten(I) * sample_in;
	end loop;

	sample_out_post_scale <= (sum_shift_ram(filterordnung-1)*30)/2**17;
	data_out <= std_logic_vector(to_unsigned(sample_out_post_scale + 2**11,12)); --unsigned output
	
end process;

end behav;